@@ -1,5 +1,5 @@
-# ************************************************************************* 
-# Copyright (c) 2014-2015, SUSE LLC
+# *************************************************************************
+# Copyright (c) 2014-2016, SUSE LLC
 # 
 # All rights reserved.
 # 
@@ -36,7 +36,8 @@ use 5.012;
 use strict;
 use warnings;
 use App::CELL qw( $CELL $log $meta $site );
-use App::Dochazka::REST::LDAP qw( populate_employee );
+use App::Dochazka::Common qw( $today init_timepiece );
+use App::Dochazka::REST::LDAP qw( ldap_search );
 use App::Dochazka::REST::Model::Shared qw( 
     cud 
     load 
@@ -224,12 +225,16 @@ This module provides the following exports:
 
 =over 
 
-=item L<nick_exists> - function
+=item L<autocreate_employee> - function
 
 =item L<eid_exists> - function
 
+=item L<get_all_sync_employees> - function
+
 =item L<list_employees_by_priv> - function
 
+=item L<nick_exists> - function
+
 =item L<noof_employees_by_priv> - function
 
 =back
@@ -238,9 +243,11 @@ This module provides the following exports:
 
 use Exporter qw( import );
 our @EXPORT_OK = qw( 
-    nick_exists 
+    autocreate_employee
     eid_exists 
+    get_all_sync_employees
     list_employees_by_priv 
+    nick_exists 
     noof_employees_by_priv 
 );
 
@@ -390,22 +397,57 @@ sub delete {
 }
 
 
-=head2 sync
+=head2 ldap_sync
 
 Sync the mapping fields to the values found in the LDAP database.
 
 =cut
 
-sub sync {
+sub ldap_sync {
     my $self = shift;
-    my $status = populate_employee( $self );
-    if ( $status->ok ) {
-        foreach my $key ( keys( %{ $site->DOCHAZKA_LDAP_MAPPING } ) ) {
-            my $value = $site->DOCHAZKA_LDAP_MAPPING->{ $key };
-            $self->set( $key, $value );
-        }
+    $log->debug( "Entering " . __PACKAGE__ . "::sync()" );
+    die "Employee nick property not populated!" unless $self->nick;
+    my $nick = $self->nick;
+
+    return $CELL->status_err( 'DOCHAZKA_LDAP_NOT_ENABLED' ) unless $site->DOCHAZKA_LDAP;
+    return $CELL->status_err(
+        'DOCHAZKA_LDAP_SYNC_PROP_FALSE',
+        args => [ $nick ],
+    ) unless $self->sync;
+    return $CELL->status_err(
+        'DOCHAZKA_LDAP_SYSTEM_USER_NOSYNC',
+        args => [ $nick ],
+    ) if grep {/^$nick/} @{ $site->DOCHAZKA_SYSTEM_USERS };
+
+    $log->debug( "About to populate $nick from LDAP" );
+
+    require Net::LDAP;
+
+    # initiate connection to LDAP server (anonymous bind)
+    my $server = $site->DOCHAZKA_LDAP_SERVER;
+    my $ldap = Net::LDAP->new( $server );
+    $log->error("$@") unless $ldap;
+    return $CELL->status_err( 'Could not connect to LDAP server' ) unless $ldap;
+
+    # get LDAP properties and stuff them into the employee object
+    my $count = 0;
+    foreach my $key ( keys( %{ $site->DOCHAZKA_LDAP_MAPPING } ) ) {
+        my $prop = $site->DOCHAZKA_LDAP_MAPPING->{ $key };
+        my $value = ldap_search( $ldap, $nick, $prop );
+        last unless $value;
+        $log->debug( "Setting $key to $value" );
+        $self->set( $key, $value );
+        $count += 1;
     }
-    return $status;
+
+    $ldap->unbind;
+
+    return $CELL->status_ok( 
+        'DOCHAZKA_LDAP_SYNC_SUCCESS',
+        args => [ $count ],
+    ) unless $count < 1;
+
+    return $CELL->status_not_ok( 'DOCHAZKA_LDAP_SYNC_FAILURE' );
 }
 
 
@@ -662,6 +704,70 @@ The following functions are not object methods.
 The following functions are exported and are not called as methods.
 
 
+=head2 autocreate_employee
+
+Takes a DBIx::Connector object and a nick - the nick is assumed not to exist in
+the Dochazka employees table. If DOCHAZKA_LDAP_AUTOCREATE is true, attempts to 
+create the employee. Returns a status object.
+
+=cut
+
+sub autocreate_employee {
+    my ( $dbix_conn, $nick ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::autocreate_employee()" );
+    my $status;
+
+    return $CELL->status_ok() if nick_exists( $dbix_conn, $nick );
+    return $CELL->status_not_ok( 'DOCHAZKA_NO_AUTOCREATE' ) unless $site->DOCHAZKA_LDAP_AUTOCREATE;
+
+    my $emp = App::Dochazka::REST::Model::Employee->spawn(
+        nick => $nick,
+        sync => 1,
+        remark => 'LDAP autocreate',
+    );
+    $status = $emp->ldap_sync();
+    return $status unless $status->ok;
+
+    my $faux_context = { 'dbix_conn' => $dbix_conn, 'current' => { 'eid' => 1 } };
+    $status = $emp->insert( $faux_context );
+    if ( $status->not_ok ) {
+        my $reason = $status->text;
+        return $CELL->status_err(
+            'DOCHAZKA_EMPLOYEE_CREATE_FAIL',
+            args => [ $nick, $reason ],
+        );
+    }
+    $log->notice( "Auto-created employee $nick, who was authenticated via LDAP" );
+
+    my $priv = $site->DOCHAZKA_LDAP_AUTOCREATE_AS;
+    if ( $priv !~ m/^(inactive)|(active)$/ ) {
+        return $CELL->status_err(
+            'DOCHAZKA_INVALID_PARAM',
+            args => [ 'DOCHAZKA_LDAP_AUTOCREATE_AS', $priv ],
+        );
+    }
+
+    # create a privhistory record (inactive/active only)
+    init_timepiece();
+    my $ph_obj = App::Dochazka::REST::Model::Privhistory->spawn(
+        eid => $emp->eid,
+        priv => $priv,
+        effective => ( $today . ' 00:00' ),
+        remark => 'LDAP autocreate',
+    );
+    $status = $ph_obj->insert( $faux_context );
+    if ( $status->not_ok ) {
+        my $reason = $status->text;
+        $status = $CELL->status_err(
+            'DOCHAZKA_AUTOCREATE_PRIV_PROBLEM',
+            args => [ $nick, $reason ],
+        );
+    }
+
+    return $status;
+}
+
+
 =head2 nick_exists
 
 See C<exists> routine in L<App::Dochazka::REST::Model::Shared>
@@ -755,6 +861,26 @@ sub noof_employees_by_priv {
 }
 
 
+=head2 get_all_sync_employees
+
+Function returns a status object. If the status is OK, the payload will contain
+a reference to an array of employee objects whose sync property is true.
+
+=cut
+
+sub get_all_sync_employees {
+    my ( $conn ) = validate_pos( @_,
+       { isa => 'DBIx::Connector' },
+    );
+    return load_multiple(
+        conn => $conn,
+        class => 'App::Dochazka::REST::Model::Employee',
+        sql => $site->SQL_EMPLOYEE_SELECT_MULTIPLE_BY_SYNC,
+        keys => [ 1 ],
+    );
+}
+
+
 =head1 AUTHOR
 
 Nathan Cutler, C<< <presnypreklad@gmail.com> >>
