@@ -143,8 +143,6 @@ them to be run separately.
 
 sub initialize_regression_test {
 
-    require App::Dochazka::REST;
-
     my $status = Web::MREST::init( 
         distro => 'App-Dochazka-REST', 
         sitedir => '/etc/dochazka-rest', 
@@ -163,9 +161,9 @@ sub initialize_regression_test {
     try {
         App::Dochazka::REST::Dispatch::init();
     } catch {
-        $status = $CELL->status_crit( 'DOCHAZKA_MASON_INIT_FAIL', args => [ $_ ] );
+        $status = $CELL->status_not_ok;
     };
-    plan skip_all => $status->text unless $status->ok;
+    plan skip_all => 'Integration testing environment not detected' unless $status->ok;
 
     note( "Check status of database server connection" );
     plan skip_all => "PostgreSQL server is unreachable" unless conn_up();
